
using Hotline.Share.Dtos;

namespace Hotline.Settings
{
    public interface ISystemAreaDomainService
    {
        Task<List<SystemArea>> GetAreaTree(int idLength = 0, string parentId = "510000");
        Task<List<Kv>> GetAreaKeyValue(int idLength = 0, string parentId = "510000");
        Task AddArea(SystemArea model,CancellationToken cancellationToken);

        Task ModifyArea(SystemArea model, CancellationToken cancellationToken);

        Task<SystemArea?> GetArea(string id, CancellationToken cancellationToken);

        Task DelArea(string id,CancellationToken cancellationToken);


        Task<string> GenerateNewAreaCodeAsync(string parentId, CancellationToken cancellationToken);
    }
}
